@@ -38,6 +38,10 @@ var app = angular.module("edge", ["ngRoute", "ngSanitize"]).config([
         template: JST["fragment-detail"],
         controller: FragmentController
       })
+      .when("/genomes/:genomeId/ssr", {
+        template: JST["genome-ssr"],
+        controller: GenomeSSRController
+      })
       .otherwise({ redirectTo: "/genomes" });
   }
 ]);
@@ -571,6 +575,7 @@ function FragmentControllerBase($scope, $routeParams, $http, $location) {
 
   $scope.annotate_error = undefined;
   $scope.addAnnotation = function(annotation) {
+    annotation["type"] = "misc"
     var data = JSON.stringify(annotation);
     $http
       .post("/edge/fragments/" + $scope.fragmentId + "/annotations/", data)
@@ -757,6 +762,7 @@ var edgeFeatureTypes = [
   "LTR",
   "LTR_retrotransposon",
   "mat_peptide",
+  "misc",
   "misc_binding",
   "misc_difference",
   "misc_feature",
@@ -862,7 +868,7 @@ function GenomeRecombinationController($scope, $routeParams, $http, $location) {
       cassette: $scope.cassette,
       homology_arm_length: $scope.homology_arm_length,
       create: false,
-      design_primers: true,
+      design_primers: false,
       primer3_opts: $scope.primer3_opts
     });
     $http
@@ -897,3 +903,74 @@ function GenomeRecombinationController($scope, $routeParams, $http, $location) {
       });
   };
 }
+
+function GenomeSSRController($scope, $routeParams, $http, $location) {
+    $scope.genomeId = $routeParams.genomeId;
+    $scope.reaction = undefined
+    $scope.new_genome_name = undefined;
+    $scope.donor = "";
+    $scope.is_donor_circular = false;
+    $scope.events = undefined;
+    $scope.waiting = false;
+    $scope.previewed = false;
+    $scope.errors = undefined;
+  
+    $http.get("/edge/genomes/" + $scope.genomeId + "/").success(function(data) {
+        $scope.genome = data;
+    });
+
+    $scope.PreviewSSR = function() {
+        console.log($scope.new_genome_name);
+        if (($scope.reaction === undefined) || ($scope.new_genome_name === undefined) || ($scope.donor === "")) {
+            $scope.previewed = undefined;
+            $scope.errors = "Please specify (1) reaction, (2) new genome name, and (3) donor sequence";
+        } else {
+            $scope.waiting = true;
+            $scope.donor = $scope.donor.replace(/\s+/g, "");
+            var data = JSON.stringify({
+                donor: $scope.donor,
+                is_donor_circular: $scope.is_donor_circular,
+                reaction: $scope.reaction,
+                genome_name: $scope.new_genome_name,
+                create: false
+            });
+            $http
+                .post("/edge/genomes/" + $scope.genomeId + "/ssr/", data)
+                .success(function(data) {
+                    $scope.events = data;
+                    $scope.waiting = false;
+                    $scope.previewed = true;
+                    $scope.errors = undefined;
+                })
+                .error(function(data, status, headers, config) {
+                    $scope.events = undefined;
+                    $scope.waiting = false;
+                    $scope.previewed = false;
+                    $scope.errors = data;
+                });
+        };
+    };
+
+    $scope.SSR = function() {
+        $scope.waiting = true;
+        $scope.donor = $scope.donor.replace(/\s+/g, "");
+        var data = JSON.stringify({
+            donor: $scope.donor,
+            is_donor_circular: $scope.is_donor_circular,
+            reaction: $scope.reaction,
+            genome_name: $scope.new_genome_name,
+            create: true
+        });
+        $http
+            .post("/edge/genomes/" + $scope.genomeId + "/ssr/", data)
+            .success(function(genome) {
+                $scope.waiting = false;
+                var url = "/genomes/" + genome.id + "/";
+                $location.path(url);
+            })
+            .error(function(data, status, headers, config) {
+                $scope.waiting = false;
+                $scope.errors = data;
+            });
+    };
+}
