@@ -22,7 +22,16 @@ from edge.tasks import build_genome_blastdb, build_genome_fragment_indices
 IS_RO_SERVER = os.getenv("RO_SERVER", False) == "True"
 
 
-def genome_export(request, genome_id):
+def genome_fasta_export(request, genome_id):
+    get_genome_or_404(genome_id)
+    io = IO(Genome.objects.get(pk=genome_id))
+    response = HttpResponse(content_type="text/plain")
+    response["Content-Disposition"] = 'attachment; filename="g%s.fa"' % genome_id
+    io.to_fasta_file(response)
+    return response
+
+
+def genome_gff_export(request, genome_id):
     get_genome_or_404(genome_id)
     io = IO(Genome.objects.get(pk=genome_id))
     response = HttpResponse(content_type="text/plain")
@@ -124,10 +133,12 @@ class RequestParser(object):
                         v = int(v)
                     elif float in field_type:
                         v = float(v)
+                    elif v is None and not required:
+                        pass
                     else:
                         raise Exception(
-                            'Field should be of type "%s", got "%s"'
-                            % (field_type, type(v))
+                            'Field "%s" should be of type "%s", got "%s"'
+                            % (name, field_type, type(v))
                         )
                 args[name] = v
         return args
@@ -182,8 +193,8 @@ class FragmentSequenceView(ViewBase):
 
 class FragmentAnnotationsView(ViewBase):
     @staticmethod
-    def to_dict(annotation):
-        return dict(
+    def to_dict(annotation, include_feature_sequence=False):
+        result = dict(
             base_first=annotation.base_first,
             base_last=annotation.base_last,
             strand=annotation.feature.strand,
@@ -201,16 +212,26 @@ class FragmentAnnotationsView(ViewBase):
             qualifiers=annotation.feature.qualifiers,
             feature_full_length=annotation.feature.length
         )
+        if include_feature_sequence:
+            result['feature']['sequence'] = annotation.feature.sequence
+        return result
 
     def on_get(self, request, fragment_id):
         q_parser = RequestParser()
         q_parser.add_argument("f", field_type=int, location="get")
         q_parser.add_argument("l", field_type=int, location="get")
         q_parser.add_argument("m", field_type=int, location="get")
+        q_parser.add_argument(
+            "include_feature_sequence",
+            field_type=str,
+            default='false',
+            location="get"
+        )
         args = q_parser.parse_args(request)
         f = args["f"]
         ll = args["l"]
         m = args["m"]
+        include_feature_sequence = args["include_feature_sequence"].lower() == "true"
 
         fragment = get_fragment_or_404(fragment_id)
         annotations = fragment.indexed_fragment().annotations(bp_lo=f, bp_hi=ll)
@@ -223,7 +244,9 @@ class FragmentAnnotationsView(ViewBase):
                 annotations = new_a
             annotations = to_return
         return [
-            FragmentAnnotationsView.to_dict(annotation) for annotation in annotations
+            FragmentAnnotationsView.to_dict(
+                annotation, include_feature_sequence=include_feature_sequence
+            ) for annotation in annotations
         ]
 
     @transaction.atomic()
@@ -569,14 +592,39 @@ class GenomePcrView(ViewBase):
 
         parser = RequestParser()
         parser.add_argument("primers", field_type=list, required=True, location="json")
+        parser.add_argument(
+            "include_feature_sequence",
+            field_type=bool,
+            required=False,
+            default=False,
+            location="json"
+        )
 
         args = parser.parse_args(request)
         primers = args["primers"]
+        include_feature_sequence = args["include_feature_sequence"]
+
         if len(primers) != 2:
             raise Exception("Expecting two primers, got %s" % (primers,))
 
-        r = pcr_from_genome(genome, primers[0], primers[1])
-        r = (r[0], [b.to_dict() for b in r[1]], [b.to_dict() for b in r[2]], r[3])
+        (
+            product, primer_a_results, primer_b_results, template_info
+        ) = pcr_from_genome(genome, primers[0], primers[1])
+        # Convert annotations in template_info to dictionary.
+        if template_info and "annotations" in template_info:
+            template_info["annotations"] = [
+                FragmentAnnotationsView.to_dict(
+                    annotation,
+                    include_feature_sequence=include_feature_sequence
+                )
+                for annotation in template_info["annotations"]
+            ]
+        r = (
+            product,
+            [b.to_dict() for b in primer_a_results],
+            [b.to_dict() for b in primer_b_results],
+            template_info,
+        )
         return r, 200
 
 
@@ -606,7 +654,7 @@ class GenomeOperationViewBase(ViewBase):
         if create is False:
             r = op_class.check(genome, **args)
             if r is None:
-                return None, 400
+                return [], 200
             return [x.to_dict() for x in r], 200
 
         else:
@@ -680,40 +728,40 @@ class GenomeCrisprDSBView(GenomeOperationViewBase):
         )
 
 
+def validate_annotations(cassette, annotations):
+    """
+    If user supplied annotations for the donor sequence, to make sure we
+    can precisely use the annotation and not leave any ambiguity, we
+    require the donor dna to not contain overhangs and backbone
+    modifications. This method returns True if that's the case, and
+    required fields for annotations exist.
+    """
+
+    if (
+        re.match(r"^[A-Za-z]+$", cassette)
+        and len(
+            [
+                a
+                for a in annotations
+                if "base_first" not in a
+                or "base_last" not in a
+                or "name" not in a
+                or "type" not in a
+                or "strand" not in a
+            ]
+        )
+        == 0
+    ):
+        return True
+
+    # if there are no supplied annotations, we don't care what format the
+    # donor sequence is
+    return len(annotations) == 0
+
+
 class GenomeRecombinationView(GenomeOperationViewBase):
     DEFAULT_HA_LENGTH = 30
 
-    @staticmethod
-    def validate_annotations(cassette, annotations):
-        """
-        If user supplied annotations for the donor sequence, to make sure we
-        can precisely use the annotation and not leave any ambiguity, we
-        require the donor dna to not contain overhangs and backbone
-        modifications. This method returns True if that's the case, and
-        required fields for annotations exist.
-        """
-
-        if (
-            re.match(r"^[A-Za-z]+$", cassette)
-            and len(
-                [
-                    a
-                    for a in annotations
-                    if "base_first" not in a
-                    or "base_last" not in a
-                    or "name" not in a
-                    or "type" not in a
-                    or "strand" not in a
-                ]
-            )
-            == 0
-        ):
-            return True
-
-        # if there are no supplied annotations, we don't care what format the
-        # donor sequence is
-        return len(annotations) == 0
-
     def parse_arguments(self, request, errors):
         from edge.recombine import RecombineOp
 
@@ -769,11 +817,7 @@ class GenomeRecombinationView(GenomeOperationViewBase):
 
         if primer3_opts is None:
             primer3_opts = {}
-        if annotations is None:
-            annotations = []
-        elif (
-            GenomeRecombinationView.validate_annotations(cassette, annotations) is False
-        ):
+        if annotations and (validate_annotations(cassette, annotations) is False):
             errors.append(
                 "Annotations failed validation: \
 please make sure donor sequence does not have overhangs \
@@ -797,3 +841,65 @@ and annotation array elements have all the required fields."
             ),
             RecombineOp,
         )
+
+
+class GenomeSSRView(GenomeOperationViewBase):
+
+    def parse_arguments(self, request, errors):
+        from edge.ssr.op import SSROp
+
+        parser = RequestParser()
+        parser.add_argument("donor", field_type=str, required=False, default=None, location="json")
+        parser.add_argument("is_donor_circular", field_type=bool, required=False, default=None,
+                            location="json")
+        parser.add_argument("reaction", field_type=str, required=True, location="json")
+        parser.add_argument("genome_name", field_type=str, required=False, default=None,
+                            location="json")
+        parser.add_argument("notes", field_type=str, required=False, default=None, location="json")
+        parser.add_argument(
+            "annotations",
+            field_type=list,
+            required=False,
+            default=None,
+            location="json",
+        )
+
+        args = parser.parse_args(request)
+        donor = args["donor"].strip() if args["donor"] else None
+        is_donor_circular = args["is_donor_circular"]
+        reaction = args["reaction"]
+        genome_name = args["genome_name"]
+        notes = args["notes"]
+        annotations = args["annotations"]
+
+        if annotations and (validate_annotations(donor, annotations) is False):
+            errors.append(
+                "Annotations failed validation: \
+please make sure donor sequence does not have overhangs \
+and annotation array elements have all the required fields."
+            )
+            return None
+
+        return (
+            dict(
+                donor=donor,
+                is_donor_circular=is_donor_circular,
+                reaction_name=reaction,
+                genome_name=genome_name,
+                notes=notes,
+                annotations=annotations,
+            ),
+            SSROp,
+        )
+
+
+class GenomeDiffView(ViewBase):
+    def on_get(self, request, child_genome_id, parent_genome_id):
+        child_genome = get_genome_or_404(child_genome_id)
+        child_genome = child_genome.indexed_genome()
+
+        try:
+            regions = child_genome.get_coordinate_diff_from_parent_genome(parent_genome_id)
+            return regions
+        except Exception as e:
+            return {'error': str(e)}
